library(gplots)
library(RColorBrewer)

bcm_root <- Sys.getenv("BCM_ROOT")
source(paste(bcm_root, "scripts/plots_functions.r", sep="/"))

model_name <- "tyson"
output_folder <- "output"
model <- load_ode_model(model_name, output_folder)

# Density overview
print("Plotting marginal densities...")
outputfn <- paste(model$output_folder, "/marginal_densities.png", sep="");
png(outputfn, width=1800, height=1000);
nx <- ceiling(sqrt(model$nvar))
ny <- ceiling(model$nvar / nx)
par(mfrow=c(ny, nx))
par(mar=c(2,2,3,2))
for (i in 1:model$nvar) {
  plot_variable_distribution(model, var_ix=i)
}
par(mfrow=c(1,1))
res <- dev.off();

# Autocorrelation plots
print("Plotting ACF...")
outputfn <- paste(model$output_folder, "/acfs.png", sep="");
png(outputfn, width=1800, height=1000);
nx <- ceiling(sqrt(model$nvar))
ny <- ceiling(model$nvar / nx)
par(mfrow=c(ny, nx))
par(mar=c(2,2,3,2))
for (i in 1:model$nvar) {
  acf(model$posterior$samples[,i], main=model$variables[i])
}
par(mfrow=c(1,1))
res <- dev.off();

# Traces
print("Plotting traces...")
outputfn <- paste(model$output_folder, "/traces.png", sep="");
png(outputfn, width=1800, height=1000);
nx <- ceiling(sqrt(model$nvar))
ny <- ceiling(model$nvar / nx)
par(mfrow=c(ny, nx))
par(mar=c(2,2,3,2))
for (i in 1:model$nvar) {
  plot(model$posterior$samples[,i], main=model$variables[i], xlab="Sample index", ylab="Value")
}
par(mfrow=c(1,1))
res <- dev.off();
