#pragma once

namespace bcm {

class Likelihood
{
public:
	virtual ~Likelihood();

	//! Evaluate the likelihood for the given values of the variables.
	//! @param threadix An index indicating which thread is calling the function.
	//! @param values The values of the variables, corresponding to the VariableSet provided to the sampler.
	//! @param logp The log likelihood value should be written into this parameter.
	//! @return True if successful, false if a non-recoverable error occured (this will cause the sampler to stop).
	virtual bool EvaluateLogProbability(size_t threadix, const Real* values, Real& logp) = 0;

	//! Provide auxiliary output variables that should be stored alongside the variable values in the posterior.
	//! This is called in two different modes:
	//! 1) At the beginning, with a NULL pointer for output_values; this is to request the number of output_variables, which should be written into the numvalues parameter.
	//! 2) Exactly once after every call to EvaluateLogProbability, to get the output_values corresponding to the variable values provided in EvaluateLogProbability.
	//! @param threadix An index indicating which thread is calling the function.
	//! @param output_values Array to which the values of the auxiliary variables should be written.
	//! @param numvalues Number of auxiliary output variables. If output_values == NULL, the number of variables should be written into this parameter.
	//! @return True if successful (even if there are no output variables), false if a non-recoverable error occured (this will cause the sampler to stop).
	virtual bool GetOutput(size_t threadix, Real* output_values, size_t& numvalues);

	//! A chance to write e.g. accumulated statistics to disk
	virtual void PostInference(const std::string& base_filename) {}
};

}
