#include "Utils.h"
#include "ProgressIndicatorConsole.h"

namespace bcm {

ProgressIndicatorConsole::ProgressIndicatorConsole()
	: prevlen(0)
	, previous_time(-std::numeric_limits<Real>::infinity())
{
}

ProgressIndicatorConsole::~ProgressIndicatorConsole()
{
}

void ProgressIndicatorConsole::OnUpdate()
{
	// Don't update more than once every half second
	if (previous_time == -std::numeric_limits<Real>::infinity()) {
		timer.Start();
		previous_time = timer.GetElapsedSeconds();
	} else {
		double new_time = timer.GetElapsedSeconds();
		if (new_time - previous_time > 0.5 || current_progress >= 1.0) {
			previous_time = new_time;
		} else {
			return;
		}
	}

	char buf[256];
	for (size_t i = 0; i < prevlen; i++) {
		buf[i] = ' ';
	}
	buf[prevlen] = 0;
	printf("\r%s", buf);

	snprintf(buf, sizeof(buf), "Progress: %6.2f%% - remaining: %s", current_progress*100.0, eta.c_str());
	for (std::map<std::string, std::string>::iterator info = current_info.begin(); info != current_info.end(); ++info) {
		strncat(buf, " - ", sizeof(buf) - strlen(buf) - 1);
		strncat(buf, info->first.c_str(), sizeof(buf) - strlen(buf) - 1);
		strncat(buf, ": ", sizeof(buf) - strlen(buf) - 1);
		strncat(buf, info->second.c_str(), sizeof(buf) - strlen(buf) - 1);
	}

	printf("\r%s", buf);
	fflush(stdout);

	prevlen = strlen(buf);
}

void ProgressIndicatorConsole::OnStop()
{
	printf("\n");
}

}
