#pragma once

#include "SMCSamplingStrategy.h"
#include "MVNRNG.h"

namespace bcm {

class SMCSamplingStrategyBlockedMCMC : public SMCSamplingStrategy
{
public:
	SMCSamplingStrategyBlockedMCMC(SamplerSMC* sampler, TaskManager* task_manager);
	virtual ~SMCSamplingStrategyBlockedMCMC();

	void SetStepsPerIteration(size_t n);
	void SetRecalculateSteps(size_t n);

protected:
	virtual bool UpdateStrategyIterationImpl(bool force_update, RNG* rng);
	virtual bool UpdateStrategyCurrentImpl(RNG* rng);
	virtual bool UpdateParticleImpl(SamplerSMC::Particle& particle, size_t ix, RNG* rng, size_t threadix);
	virtual bool CalculateParticleWeightImpl(SamplerSMC::Particle& particle, size_t ix, size_t threadix);

	// Settings
	size_t recalculate_steps;
	size_t num_mcmc_steps_per_iteration;
	Real target_acceptance_rate;

	// Run-time variables
	struct Block
	{
		std::vector<size_t> variable_indices;
		MVNRNG mvn_sampler;	// For full covariance sampling
		VectorReal sigma;	// For diagonal covariance sampling
		Real scale;
	};
	std::vector<Block> sampler_blocks;
	VectorReal previous_scale;

	struct AsyncParams {
		VectorReal proposed_var_values;
		std::vector<size_t> num_proposed;
		std::vector<size_t> num_accepted;
	};
	std::vector<AsyncParams> async;

	size_t update_step;
};

}
