#pragma once

#include "SMCSamplingStrategy.h"
#include "MVNRNG.h"

namespace bcm {

class SMCSamplingStrategyMCMC : public SMCSamplingStrategy
{
public:
	SMCSamplingStrategyMCMC(SamplerSMC* sampler, TaskManager* task_manager);
	virtual ~SMCSamplingStrategyMCMC();

	void SetGlobal();
	void SetGlobalCovariance();
	void SetLocal(size_t num_nearby);
	void SetStepsPerIteration(size_t n);
	void SetRecalculateSteps(size_t n);

protected:
	virtual bool UpdateStrategyIterationImpl(bool force_update, RNG* rng);
	virtual bool UpdateStrategyCurrentImpl(RNG* rng);
	virtual bool UpdateParticleImpl(SamplerSMC::Particle& particle, size_t ix, RNG* rng, size_t threadix);
	virtual bool CalculateParticleWeightImpl(SamplerSMC::Particle& particle, size_t ix, size_t threadix);

	void UpdateStrategyLocalMCMC(void* user, size_t thread);
	
	enum EProposalType {
		PT_GlobalEmpiricalVariance,
		PT_GlobalEmpiricalCovariance,
		PT_LocalEmpiricalVariance,
	};

	// Settings
	EProposalType proposal_type;
	size_t num_nearby_particles;
	size_t recalculate_steps;
	size_t num_mcmc_steps_per_iteration;
	Real target_acceptance_rate;

	// Run-time variables
	// For global diagonal
	VectorReal empirical_mean;
	VectorReal empirical_variance;
	VectorReal proposal_sigma;

	// For global covariance
	MatrixReal empirical_covariance;
	MVNRNG mvn_sampler;

	// For local diagonal
	struct LocalProposal {
		VectorReal proposal_sigma;
	};
	std::vector<LocalProposal> local_proposals;

	struct AsyncParams {
		VectorReal proposed_var_values;
		size_t num_proposed;
		size_t num_accepted;
	};
	std::vector<AsyncParams> async;

	size_t update_step;
	Real proposal_scale;
};

}
