#include "Utils.h"
#include "ExperimentalCondition.h"

ExperimentalCondition::ExperimentalCondition()
{
}

ExperimentalCondition::~ExperimentalCondition()
{
}

bool ExperimentalCondition::Load(const boost::property_tree::ptree& xml_node, const bcm::NetCDFDataFile* datafile)
{
	// Parse XML
	try {
		std::string type = xml_node.get<std::string>("<xmlattr>.type");
		if (type == "variable") {
			Name = xml_node.get<std::string>("<xmlattr>.variable_name");

			std::string valuestr = xml_node.get<std::string>("<xmlattr>.value");

			if (valuestr.front() == '\'' && valuestr.back() == '\'') {
				LOGERROR("Not implemented");
				return false;
			} else {
				size_t indexing_start = valuestr.find_first_of('[');
				if (indexing_start != std::string::npos) {
					// Looks like a data value lookup; try to find it.
					size_t indexing_end = valuestr.find_first_of(']');
					if (indexing_end == std::string::npos) {
						LOGERROR("Syntax error in valuestr: missing closing bracket for indexing");
						return false;
					}

					std::string data_name_without_index = valuestr.substr(0, indexing_start);
					std::vector<std::string> groupandname;
					bcm::tokenize(data_name_without_index, groupandname, "/");
					if (groupandname.size() != 2) {
						LOGERROR("Syntax error in valuestr: need group and variable name separated by a /");
					}

					std::string indexingstr = valuestr.substr(indexing_start+1, indexing_end - indexing_start - 1);
					std::vector<std::string> indexing_parts;
					bcm::tokenize(indexingstr, indexing_parts, ",");

					std::vector<size_t> data_indices;
					for (size_t i = 0; i < indexing_parts.size(); i++) {
						if (indexing_parts[i].front() == '\'' && indexing_parts[i].back() == '\'') {
							std::string val = indexing_parts[i].substr(1, indexing_parts[i].size() - 2);
							std::string dimname;
							if (!datafile->GetDimensionName(groupandname[0], groupandname[1], i, dimname)) {
								return false;
							}

							size_t ix;
							if (!datafile->GetDimensionIx(groupandname[0], dimname, val, &ix)) {
								return false;
							}
							data_indices.push_back(ix);
						} else {
							size_t ix;
							try {
								ix = boost::lexical_cast<size_t>(indexing_parts[i]);
							} catch( boost::bad_lexical_cast const& ) {
								LOGERROR("Error converting index to a number");
								return false;
							}
							data_indices.push_back(ix);
						}
					}

					if (data_indices.size() == 1) {
						if (!datafile->GetValue(groupandname[0], groupandname[1], data_indices[0], &Value)) {
							return false;
						}
					} else if (data_indices.size() == 2) {
						if (!datafile->GetValue(groupandname[0], groupandname[1], data_indices[0], data_indices[1], &Value)) {
							return false;
						}
					} else if (data_indices.size() == 3) {
						if (!datafile->GetValue(groupandname[0], groupandname[1], data_indices[0], data_indices[1], data_indices[2], &Value)) {
							return false;
						}
					} else {
						LOGERROR("Unsupported");
						return false;
					}
				} else {
					// Probably a constant value, try to cast to a number
					try {
						Value = boost::lexical_cast<Real>(valuestr);
					} catch( boost::bad_lexical_cast const& ) {
						LOGERROR("Error converting value \"%s\" to a number", valuestr.c_str());
						return false;
					}
				}
			}
		} else {
			LOGERROR("Unknown condition type \"%s\"", type.c_str());
			return false;
		}
	} catch (boost::property_tree::ptree_error &e) {
		LOGERROR("Error parsing likelihood file: %s", e.what());
		return false;
	}

	return true;
}
