#include "Utils.h"
#include "LikelihoodSingleTimepoint.h"
#include "SBMLModel.h"

LikelihoodSingleTimepoint::LikelihoodSingleTimepoint()
	: NumReplicates(0)
	, Timepoint(std::numeric_limits<Real>::quiet_NaN())
{
}

LikelihoodSingleTimepoint::~LikelihoodSingleTimepoint()
{
}

bool LikelihoodSingleTimepoint::Load(const boost::property_tree::ptree& xml_node, const bcm::NetCDFDataFile* datafile)
{
	if (!LikelihoodPart::Load(xml_node, datafile)) {
		return false;
	}

	if (LikelihoodFunction == LF_Delta) {
		LOGERROR("Cannot have delta likelihood function on a timepoint (cannot force trajectory to certain values in the ODE solver)");
		return false;
	}
	
	try {
		Timepoint  = xml_node.get<Real>("<xmlattr>.time");
	} catch (boost::property_tree::ptree_error &e) {
		LOGERROR("Error parsing likelihood file: %s", e.what());
		return false;
	}

	// Load the data into memory
	if (!datafile->GetDimensionSize(DataGroup, DataNameWithoutIndex + "_replicates", &NumReplicates)) {
		return false;
	}

	Data.resize(NumReplicates);
	for (size_t ri = 0; ri < NumReplicates; ri++) {
		if (!LookupDataValue1D(datafile, ri, &Data[ri])) {
			return false;
		}
	}

	return true;
}

bool LikelihoodSingleTimepoint::SetVariableSet(const bcm::VariableSet* variables)
{
	if (!LikelihoodPart::SetVariableSet(variables)) {
		return false;
	}

	return true;
}

bool LikelihoodSingleTimepoint::GetRequiredTimepoints(std::vector<Real>& timepoints)
{
	timepoints.resize(1);
	timepoints[0] = Timepoint;
	return true;
}

bool LikelihoodSingleTimepoint::EvaluateLogProbability(size_t threadix, const Real* values, SBMLModel* model, std::vector<Real> simulated_time, Real& logp)
{
	VectorReal simulated_trajectory;
	if (!model->GetOutput(SpeciesName, simulated_trajectory, threadix)) {
		return false;
	}

	bool found = false;
	logp = 0.0;
	for (size_t sti = 0; sti < simulated_time.size(); sti++) {
		if (simulated_time[sti] == Timepoint) {
			for (size_t ri = 0; ri < NumReplicates; ri++) {
				logp += EvaluateLogPdf(values, simulated_trajectory(sti), Data[ri]);
			}
			found = true;
			break;
		}
	}
	ASSERT(found);

	return true;
}
