#include "Utils.h"
#include "Prediction.h"
#include "SBMLModel.h"

#include <boost/property_tree/xml_parser.hpp>

Prediction::Prediction()
{
}

Prediction::~Prediction()
{
}

bool Prediction::Load(const std::string& filename)
{
	boost::property_tree::ptree pt;
	try {
		boost::property_tree::read_xml(filename, pt);
	} catch (boost::property_tree::xml_parser_error &e) {
		LOGERROR("Error loading prediction file: %s", e.what());
		return false;
	}
	
	try {
		boost::property_tree::ptree prediction_node = pt.get_child("prediction");

		model_filename = prediction_node.get_child("model").get<std::string>("<xmlattr>.file");

		boost::property_tree::ptree timedesc = prediction_node.get_child("time_description");
		Real start_time = timedesc.get<Real>("<xmlattr>.start_time");
		Real end_time = timedesc.get<Real>("<xmlattr>.end_time");
		size_t num_timepoints = timedesc.get<size_t>("<xmlattr>.num_timepoints");

		if (num_timepoints < 2) {
			LOGERROR("Need at least two timepoints");
			return false;
		}

		timepoints.resize(num_timepoints);
		timepoints[0] = start_time;
		timepoints[num_timepoints-1] = end_time;
		for (size_t i = 1; i < num_timepoints - 1; i++) {
			timepoints[i] = start_time + (i / (Real)(num_timepoints - 1)) * (end_time - start_time);
		}
		
		BOOST_FOREACH(const boost::property_tree::ptree::value_type& var, prediction_node.get_child("")) {
			if (var.first == "condition") {
				Condition c;
				c.variable = var.second.get<std::string>("<xmlattr>.variable_name");
				c.value = var.second.get<Real>("<xmlattr>.value");
				conditions.push_back(c);
			}
		}
	} catch (boost::property_tree::ptree_error &e) {
		LOGERROR("Error parsing prediction file: %s", e.what());
		return false;
	}

	return true;
}

bool Prediction::SetConditionsAtModel(SBMLModel* model, size_t threadix)
{
	model->ResetForcedVariables(threadix);
	for (std::vector<Condition>::iterator ci = conditions.begin(); ci != conditions.end(); ++ci) {
		if (!model->SetForcedVariable(ci->variable, ci->value, threadix)) {
			return false;
		}
	}
	return true;
}

const std::string& Prediction::GetModelFilename() const
{
	return model_filename;
}

const std::vector<Real>& Prediction::GetSimulatedTimepoints() const
{
	return timepoints;
}
