#include "Utils.h"
#include "Clustering.h"

extern "C" {
#include "../../dependencies/cluster-1.52a/src/cluster.h"
}

namespace bcm {

size_t find_cluster(Node* nodelist, std::vector< std::set<size_t> >& clusters, int i)
{
	int el = i;
	while (el < 0) {
		size_t ix = (-el) - 1;
		if (nodelist[ix].left >= 0) {
			el = nodelist[ix].left;
			break;
		}
		if (nodelist[ix].right >= 0) {
			el = nodelist[ix].right;
			break;
		}
		el = nodelist[ix].right;
	}
	for (size_t ci = 0; ci < clusters.size(); ci++) {
		if (clusters[ci].find(el) != clusters[ci].end()) {
			return ci;
		}
	}
	return std::numeric_limits<size_t>::max();
}

bool TreeCluster(const MatrixReal& distance, std::vector< std::set<size_t> >& clusters, Real cut_height)
{
	if (distance.cols() < 1) {
		return false;
	}

#if 0
	MatrixReal dcopy = distance;
	std::vector<Real*> pointers;
	pointers.resize(dcopy.cols(), NULL);
	for (size_t i = 0; i < (size_t)dcopy.cols(); i++) {
		pointers[i] = dcopy.data() + i * dcopy.rows();
	}
#else
	Eigen::MatrixXd dcopy = distance.cast<double>();
	std::vector<double*> pointers;
	pointers.resize(dcopy.cols(), NULL);
	for (size_t i = 0; i < (size_t)dcopy.cols(); i++) {
		pointers[i] = dcopy.data() + i * dcopy.rows();
	}
#endif

	Node* nodelist = treecluster((int)dcopy.cols(), (int)dcopy.cols(), NULL, NULL, NULL, 0, 0, 'm', &pointers[0]);
	
	clusters.clear();

	for (int i = 0; i < dcopy.cols() - 1; i++) {
		if (nodelist[i].distance < cut_height) {
			if (nodelist[i].left >= 0) {
				if (nodelist[i].right >= 0) {
					// New cluster
					clusters.push_back(std::set<size_t>());
					clusters.back().insert(nodelist[i].left);
					clusters.back().insert(nodelist[i].right);
				} else {
					// Add left to cluster containing right
					size_t ci = find_cluster(nodelist, clusters, nodelist[i].right);
					if (ci == std::numeric_limits<size_t>::max()) {
						LOGERROR("Error in clustering");
						return false;
					}
					clusters[ci].insert(nodelist[i].left);
				}
			} else {
				size_t ci = find_cluster(nodelist, clusters, nodelist[i].left);
				if (nodelist[i].right >= 0) {
					// Add right to cluster containing left
					clusters[ci].insert(nodelist[i].right);
					if (ci == std::numeric_limits<size_t>::max()) {
						LOGERROR("Error in clustering");
						return false;
					}
				} else {
					// Merge right into left
					size_t ci2 = find_cluster(nodelist, clusters, nodelist[i].right);
					clusters[ci].insert(clusters[ci2].begin(), clusters[ci2].end());
					clusters.erase(clusters.begin() + ci2);
				}
			}
		} else {
			if (nodelist[i].left >= 0) {
				clusters.push_back(std::set<size_t>());
				clusters.back().insert(nodelist[i].left);
			}
			if (nodelist[i].right >= 0) {
				clusters.push_back(std::set<size_t>());
				clusters.back().insert(nodelist[i].right);
			}
		}
	}

	return true;
}

void KCluster(int k, size_t nsamples, size_t dim, const Real** data, const Real* weights, int npass, int* assignment, Real* error)
{
	int ifound;
	std::vector<int*> mask(nsamples);
	for (size_t i = 0; i < nsamples; i++) {
		mask[i] = new int[dim];
		for (size_t j = 0; j < dim; j++) {
			mask[i][j] = 1;
		}
	}
	//TODO - single precision support
	kcluster(k, (int)nsamples, (int)dim, const_cast<Real**>(data), mask.data(), const_cast<Real*>(weights), 0, npass, 'a', 'e', assignment, error, &ifound);
	for (size_t i = 0; i < nsamples; i++) {
		delete[] mask[i];
	}
}

}
