#pragma once

#ifdef BCM_INCLUDE_NETCDF

#include <netcdf>

namespace bcm {

class NetCDFDataFile
{
public:
	NetCDFDataFile();
	~NetCDFDataFile();

	bool Open(const std::string& filename, netCDF::NcFile::FileMode mode);
	void Close();
	bool IsOpen();
	
	bool GetValue(const std::string& group_name, const std::string& variable_name, size_t dim1ix, Real* value) const;
	bool GetValue(const std::string& group_name, const std::string& variable_name, size_t dim1ix, size_t dim2ix, Real* value) const;
	bool GetValue(const std::string& group_name, const std::string& variable_name, size_t dim1ix, size_t dim2ix, size_t dim3ix, Real* value) const;
	bool GetValue(const std::string& group_name, const std::string& variable_name, size_t dim1ix, size_t dim2ix, size_t dim3ix, size_t dim4ix, Real* value) const;
	bool GetValue(const std::string& group_name, const std::string& variable_name, size_t dim1ix, std::string& value) const;
	bool GetValue(const std::string& group_name, const std::string& variable_name, const std::string& dim1value, Real* value) const;
	bool GetValue(const std::string& group_name, const std::string& variable_name, const std::string& dim1value, const std::string& dim2value, Real* value) const;
	bool GetValue(const std::string& group_name, const std::string& variable_name, const std::string& dim1value, const std::string& dim2value, const std::string& dim3value, Real* value) const;
	
	bool GetValues(const std::string& group_name, const std::string& variable_name, size_t dim1ix, size_t count, VectorReal& values) const;
	bool GetValuesDim1(const std::string& group_name, const std::string& variable_name, size_t dim1ix, size_t dim2ix, size_t count, VectorReal& values) const;
	bool GetValuesDim2(const std::string& group_name, const std::string& variable_name, size_t dim1ix, size_t dim2ix, size_t count, VectorReal& values) const;
	bool GetValuesDim1(const std::string& group_name, const std::string& variable_name, size_t dim1ix, size_t dim2ix, size_t dim3ix, size_t count, VectorReal& values) const;
	bool GetValuesDim2(const std::string& group_name, const std::string& variable_name, size_t dim1ix, size_t dim2ix, size_t dim3ix, size_t count, VectorReal& values) const;
	bool GetValuesDim3(const std::string& group_name, const std::string& variable_name, size_t dim1ix, size_t dim2ix, size_t dim3ix, size_t count, VectorReal& values) const;
	bool GetValues(const std::string& group_name, const std::string& variable_name, std::vector<size_t> ix, std::vector<size_t> count, Real* values) const;

	template<typename T>
	bool GetValue(const std::string& group_name, const std::string& variable_name, const std::vector<size_t>& ix, T* value) const
	{
		try {
			netCDF::NcGroup group = GetGroup(group_name);
			netCDF::NcVar var = group.getVar(variable_name);
			var.getVar(ix, value);
			return true;
		} catch(netCDF::exceptions::NcException& e) {
			LOGERROR("Error retrieving value for %s/%s: %s", group_name.c_str(), variable_name.c_str(), e.what());
			return false;
		}
	}

	template<typename T>
	bool GetValue(const std::string& group_name, const std::string& variable_name, const std::vector<std::string>& ix, T* value) const
	{
		try {
			netCDF::NcGroup group = GetGroup(group_name);
			netCDF::NcVar var = group.getVar(variable_name);

			std::vector<size_t> ixvalue;
			ixvalue.resize(ix.size());
			for (size_t i = 0; i < ix.size(); i++) {
				if (!GetDimensionIx(group, var.getDim((int)i), ix[i], &ixvalue[i])) {
					return false;
				}
			}
			var.getVar(ixvalue, value);
			return true;
		} catch(netCDF::exceptions::NcException& e) {
			LOGERROR("Error retrieving value for %s/%s: %s", group_name.c_str(), variable_name.c_str(), e.what());
			return false;
		}
	}

	bool VariableExists(const std::string& group_name, const std::string& variable_name) const;
	bool GetDimensionCount(const std::string& group_name, const std::string& variable_name, size_t* count) const;
	bool GetDimensionName(const std::string& group_name, const std::string& variable_name, size_t dimix, std::string& dimname) const;
	bool GetDimensionSize(const std::string& group_name, const std::string& dimname, size_t* size) const;
	bool GetDimensionIx(const std::string& group_name, const std::string& dimname, const std::string& dimvalue, size_t* ix) const;

	bool GetAttribute(const std::string& group_name, const std::string& attribute_name, Real& value) const;
	
	bool CreateGroup(const std::string& group_name);
	bool CreateDimension(const std::string& group_name, const std::string& dimname, const std::vector<std::string>& dimvalues);
	bool CreateDimension(const std::string& group_name, const std::string& dimname, const std::vector<unsigned int>& dimvalues);
	bool CreateDimension(const std::string& group_name, const std::string& dimname, const std::vector<Real>& dimvalues);
	bool CreateDimension(const std::string& group_name, const std::string& dimname, const VectorReal& dimvalues);
	bool CreateVariable(const std::string& group_name, const std::string& variable_name, const std::string& dim1name);
	bool CreateVariable(const std::string& group_name, const std::string& variable_name, const std::string& dim1name, const std::string& dim2name);
	bool CreateVariable(const std::string& group_name, const std::string& variable_name, const std::string& dim1name, const std::string& dim2name, const std::string& dim3name);
	bool CreateVariable(const std::string& group_name, const std::string& variable_name, const std::string& dim1name, const std::string& dim2name, const std::string& dim3name, const std::string& dim4name);

	bool PutValue(const std::string& group_name, const std::string& variable_name, const std::string& dim1value, Real value);
	bool PutValue(const std::string& group_name, const std::string& variable_name, const std::string& dim1value, const std::string& dim2value, Real value);
	bool PutValue(const std::string& group_name, const std::string& variable_name, const std::string& dim1value, size_t dim2ix, Real value);
	bool PutValue(const std::string& group_name, const std::string& variable_name, size_t dim1ix, unsigned int value);
	bool PutValue(const std::string& group_name, const std::string& variable_name, size_t dim1ix, Real value);
	bool PutValue(const std::string& group_name, const std::string& variable_name, size_t dim1ix, size_t dim2ix, Real value);
	bool PutValue(const std::string& group_name, const std::string& variable_name, size_t dim1ix, size_t dim2ix, size_t dim3ix, Real value);
	bool PutValue(const std::string& group_name, const std::string& variable_name, size_t dim1ix, const std::string& dim2value, size_t dim3ix, Real value);

	bool PutValues(const std::string& group_name, const std::string& variable_name, size_t dim1ix, const VectorReal& values);
	bool PutValuesDim1(const std::string& group_name, const std::string& variable_name, size_t dim1ix, size_t dim2ix, const VectorReal& values);
	bool PutValuesDim2(const std::string& group_name, const std::string& variable_name, size_t dim1ix, size_t dim2ix, const VectorReal& values);
	bool PutValuesDim1(const std::string& group_name, const std::string& variable_name, size_t dim1ix, size_t dim2ix, size_t dim3ix, const VectorReal& values);
	bool PutValuesDim2(const std::string& group_name, const std::string& variable_name, size_t dim1ix, size_t dim2ix, size_t dim3ix, const VectorReal& values);
	bool PutValuesDim3(const std::string& group_name, const std::string& variable_name, size_t dim1ix, size_t dim2ix, size_t dim3ix, const VectorReal& values);
	bool PutValues(const std::string& group_name, const std::string& variable_name, const std::vector<size_t>& ix, const std::vector<size_t>& count, const Real* values);

	bool GetSubgroups(const std::string& group_name, std::vector<std::string>& groups);

private:
	netCDF::NcGroup GetGroup(const std::string& group_name) const;
	void CreateDimensionalVariable(netCDF::NcGroup group, netCDF::NcDim dim, const std::vector<std::string>& dimvalues);
	void CreateDimensionalVariable(netCDF::NcGroup group, netCDF::NcDim dim, const std::vector<unsigned int>& dimvalues);
	void CreateDimensionalVariable(netCDF::NcGroup group, netCDF::NcDim dim, const std::vector<Real>& dimvalues);
	void CreateDimensionalVariable(netCDF::NcGroup group, netCDF::NcDim dim, const VectorReal& dimvalues);
	bool GetDimensionIx(netCDF::NcGroup group, netCDF::NcDim dim, const std::string& dimvalue, size_t* ix) const;
	netCDF::NcFile* File;
};

}

#endif
