#pragma once

namespace bcm {

typedef boost::function<void (void*, size_t)> TTask;

class TaskManager
{
public:
	TaskManager(size_t numthreads);
	~TaskManager();

	uint64 AddTask(TTask task, void* userdata);
	void WaitTask(uint64 task);

	inline size_t GetNumThreads() const { return Threads.size(); }

private:
	struct Task
	{
		TTask Delegate;
		void* UserData;
		uint64 Handle;
	};
	void WorkerFunction(size_t threadIndex);
	friend void worker(TaskManager* manager, size_t threadIndex);

	boost::mutex StartMutex;
	boost::mutex FinishMutex;
	boost::condition_variable StartCondition;
	boost::condition_variable FinishCondition;
	bool Active;

	std::vector<boost::thread*> Threads;
	std::list<Task> WaitingTasks;
	std::list<Task> FinishedTasks;
	uint64 NextHandle;
};

}
