#include "Utils.h"
#include "Correlation.h"

namespace bcm {

void sift(VectorReal& x, size_t start, size_t end)
{
	Real v = x[start];

	while (start <= end/2) {
		size_t j = 2 * start;
		if (j < end && x[j] < x[j+1]) {
			j++;
		}
		if (!(v < x[j])) {
			break;
		}
		x[start] = x[j];
		start = j;
	}

	x[start] = v;
}

void sort(VectorReal& x)
{
	if (x.size() == 0) {
		return;
	}

	size_t end = x.size() - 1;
	size_t start = end / 2 + 1;

	do {
		start--;
		sift(x, start, end);
	} while (start > 0);

	while (end > 0) {
		std::swap(x[0], x[end]);
		end--;
		sift(x, 0, end);
	}
}

void sift(VectorReal& x, VectorReal& aux, size_t start, size_t end)
{
	Real v = x[start];
	Real vaux = aux[start];

	while (start <= end/2) {
		size_t j = 2 * start;
		if (j < end && x[j] < x[j+1]) {
			j++;
		}
		if (!(v < x[j])) {
			break;
		}
		x[start] = x[j];
		aux[start] = aux[j];
		start = j;
	}

	x[start] = v;
	aux[start] = vaux;
}

void sort(VectorReal& x, VectorReal& aux)
{
	if (x.size() == 0) {
		return;
	}

	size_t end = x.size() - 1;
	size_t start = end / 2 + 1;

	do {
		start--;
		sift(x, aux, start, end);
	} while (start > 0);

	while (end > 0) {
		std::swap(x[0], x[end]);
		std::swap(aux[0], aux[end]);
		end--;
		sift(x, aux, 0, end);
	}
}

void sift(VectorReal& x, VectorReal& aux1, VectorReal& aux2, size_t start, size_t end)
{
	Real v = x[start];
	Real vaux1 = aux1[start];
	Real vaux2 = aux2[start];

	while (start <= end/2) {
		size_t j = 2 * start;
		if (j < end && x[j] < x[j+1]) {
			j++;
		}
		if (!(v < x[j])) {
			break;
		}
		x[start] = x[j];
		aux1[start] = aux1[j];
		aux2[start] = aux2[j];
		start = j;
	}

	x[start] = v;
	aux1[start] = vaux1;
	aux2[start] = vaux2;
}

void sort(VectorReal& x, VectorReal& aux1, VectorReal& aux2)
{
	if (x.size() == 0) {
		return;
	}

	size_t end = x.size() - 1;
	size_t start = end / 2 + 1;

	do {
		start--;
		sift(x, aux1, aux2, start, end);
	} while (start > 0);

	while (end > 0) {
		std::swap(x[0], x[end]);
		std::swap(aux1[0], aux1[end]);
		std::swap(aux2[0], aux2[end]);
		end--;
		sift(x, aux1, aux2, 0, end);
	}
}

bool ParseVectorFromString(const std::string& str, VectorReal& v)
{
	std::vector<std::string> vec;
	bcm::tokenize(str, vec, ";");
	v.resize(vec.size());
	for (size_t i = 0; i < vec.size(); i++) {
		Real value = boost::lexical_cast<Real>(vec[i]);
		v(i) = value;
	}
	return true;
}

}
