#include "Utils.h"
#include <boost/test/unit_test.hpp>

#include "ProbabilityDistributions.h"

BOOST_AUTO_TEST_CASE(PdfBeta)
{
	BOOST_CHECK_EQUAL(bcm::PdfBeta(-0.5, 1, 1), 0);
	BOOST_CHECK_EQUAL(bcm::PdfBeta( 0  , 1, 1), 1);
	BOOST_CHECK_CLOSE(bcm::PdfBeta( 0.2, 1, 1), 1, 1e-6);
	BOOST_CHECK_EQUAL(bcm::PdfBeta( 1  , 1, 1), 1);
	BOOST_CHECK_EQUAL(bcm::PdfBeta( 1.2, 1, 1), 0);
	
	BOOST_CHECK_EQUAL(bcm::PdfBeta(-0.5, 2, 2), 0);
	BOOST_CHECK_EQUAL(bcm::PdfBeta( 0  , 2, 2), 0);
	BOOST_CHECK_CLOSE(bcm::PdfBeta( 0.2, 2, 2), 0.96, 1e-6);
	BOOST_CHECK_EQUAL(bcm::PdfBeta( 1  , 2, 2), 0);
	BOOST_CHECK_EQUAL(bcm::PdfBeta( 1.2, 2, 2), 0);
}

BOOST_AUTO_TEST_CASE(PdfExponential)
{
	BOOST_CHECK_EQUAL(bcm::PdfExponential(-0.5, 1), 0);
	BOOST_CHECK_CLOSE(bcm::PdfExponential( 0  , 1), 1, 1e-6);
	BOOST_CHECK_CLOSE(bcm::PdfExponential( 0.2, 1), 0.81873075307798182099, 1e-6);
	BOOST_CHECK_CLOSE(bcm::PdfExponential( 2.9, 1), 0.055023220056407230738, 1e-6);
	
	BOOST_CHECK_EQUAL(bcm::PdfExponential(-0.5, 2.1), 0);
	BOOST_CHECK_CLOSE(bcm::PdfExponential( 0  , 2.1), 2.1, 1e-6);
	BOOST_CHECK_CLOSE(bcm::PdfExponential( 0.2, 2.1), 1.379798321611619194, 1e-6);
	BOOST_CHECK_CLOSE(bcm::PdfExponential( 2.9, 2.1), 0.0047573587211100767588, 1e-6);
}

BOOST_AUTO_TEST_CASE(PdfGamma)
{
	BOOST_CHECK_EQUAL(bcm::PdfGamma(-0.5, 1, 1), 0);
	BOOST_CHECK_CLOSE(bcm::PdfGamma( 0  , 1, 1), 1, 1e-6);
	BOOST_CHECK_CLOSE(bcm::PdfGamma( 0.2, 1, 1), 0.81873075307798182099, 1e-6);
	BOOST_CHECK_CLOSE(bcm::PdfGamma( 2.9, 1, 1), 0.055023220056407230738, 1e-6);
	
	BOOST_CHECK_EQUAL(bcm::PdfGamma(-0.5, 2.1, 4.3), 0);
	BOOST_CHECK_CLOSE(bcm::PdfGamma( 0  , 2.1, 4.3), 0, 1e-6);
	BOOST_CHECK_CLOSE(bcm::PdfGamma( 0.2, 2.1, 4.3), 0.0072596575709185182795, 1e-6);
	BOOST_CHECK_CLOSE(bcm::PdfGamma( 2.9, 2.1, 4.3), 0.07340479047437008242, 1e-6);
}

BOOST_AUTO_TEST_CASE(PdfNormal)
{
	BOOST_CHECK_CLOSE(bcm::PdfNormal( 0  , 0, 1), 0.39894228040143270286, 1e-6);
	BOOST_CHECK_CLOSE(bcm::PdfNormal(-1.2, 0, 1), 0.19418605498321295233, 1e-6);
	BOOST_CHECK_CLOSE(bcm::PdfNormal( 1.5, 0, 1), 0.12951759566589174377, 1e-6);

	BOOST_CHECK_CLOSE(bcm::PdfNormal(   0  , 1.8, 14.2), 0.027869716269854367258, 1e-6);
	BOOST_CHECK_CLOSE(bcm::PdfNormal(   1.8, 1.8, 14.2), 0.028094526788833291964, 1e-6);
	BOOST_CHECK_CLOSE(bcm::PdfNormal(-102.2, 1.8, 14.2), 6.3213666207045896396e-14, 1e-6);
	BOOST_CHECK_CLOSE(bcm::PdfNormal(  16.0, 1.8, 14.2), 0.017040191867545309568, 1e-6);
}
